@@ -1,5 +1,7 @@
 """The Board class for the bridgeobjects package."""
 
+from typing import Dict, List, Union, Tuple
+
 from .card import Card
 from .hand import Hand
 from .auction import Auction
@@ -26,44 +28,46 @@ class Board(object):
             (list) a list of hand objects.
     """
 
-    def __init__(self, identifier='', hands=None):
+    def __init__(self, identifier: Union[str, None]='', hands: Union[List[Hand], None]=None):
         # identifier/board number: a positive integer (as string)  under PBN but
         # can be any character combination under RPN.
         if identifier is None:
             identifier = ''
         self.identifier = identifier
 
-        self.vulnerable = None  # must be in VULNERABLE.
+        self.vulnerable = ''  # must be in VULNERABLE.
         if hands is None:
-            self._hands, self._hands_by_index, self._hands_by_seat  = {}, {}, {}
+            self._hands: Dict[object, Hand] = dict()
+            self._hands_by_index: Dict[int, Hand] = dict()
+            self._hands_by_seat: Dict[str, Hand]  = dict()
         else:
-            self._hands, self._hands_by_index, self._hands_by_seat  = self._set_hands(hands)
+            (self._hands, self._hands_by_index, self._hands_by_seat)  = self._set_hands(hands)
 
-        self._dealer = None    # must be in SEATS
+        self._dealer = ''    # must be in SEATS
         self._dealer_index = -1
         self.north = ""
         self.south = ""
         self.west = ""
         self.east = ""
         self.result = ""
-        self._declarer = None    # must be in SEATS
+        self._declarer = ''    # must be in SEATS
         self._declarer_index = -1
         self._contract = Contract()
         self.auction = Auction()
-        self.tricks = []
+        self.tricks: List[Trick] = []
         self.NS_tricks = 0
         self.EW_tricks = 0
-        self.play_notes = {}
+        self.play_notes: Dict[str, str] = {}
         self.description = ""
 
-    def _set_hands(self, hands):
+    def _set_hands(self, hands: List[Hand]) -> Tuple[Dict[object, Hand], Dict[int, Hand], Dict[str, Hand]]:
         """Return the hands as a dict."""
         # If hands is a list then 0 is the North hand and proceed clockwise.
         # If hands is a dict, then determine if the keys are numeric (position) or
         # alpha (SEATS) and proceed accordingly.
-        hand_dict = {}
-        hands_by_index = {}
-        hands_by_seat = {}
+        hand_dict: Dict[object, Hand] = dict()
+        hands_by_index: Dict[int, Hand] = dict()
+        hands_by_seat: Dict[str, Hand] = dict()
         # hands is a list
         if isinstance(hands, list):
             for key, hand in enumerate(hands):
@@ -103,29 +107,21 @@ class Board(object):
                     hand_dict[key] = hand
                     hand_dict[SEATS[key]] = hand
                     break
-        return hand_dict, hands_by_index, hands_by_seat
+        return (hand_dict, hands_by_index, hands_by_seat)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """Return a repr string for the object."""
         if self._hands_by_index:
             return f'Board("{self._identifier}", {self._hands_by_index})'
         else:
             return f'Board("{self._identifier}")'
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Return a str string for the object."""
         return f'Board("{self._identifier}")'
 
-    # @staticmethod
-    # def _create_tricks():
-    #     """Return a dict of the board's tricks."""
-    #     tricks = {}
-    #     for index in range(13):
-    #         tricks[index] = Trick(index)
-    #     return tricks
-
     @property
-    def vulnerable(self):
+    def vulnerable(self) -> str:
         """
         Return the board's vulnerable status as a string.
         Valid values are in the constant VULNERABLE:
@@ -134,7 +130,7 @@ class Board(object):
         return self._vulnerable
 
     @vulnerable.setter
-    def vulnerable(self, value):
+    def vulnerable(self, value: str):
         """Set the vulnerable property to an allowed value."""
         if value and value not in VULNERABLE:
             raise ValueError('f{value} not a valid value for vulnerable')
@@ -143,7 +139,7 @@ class Board(object):
         self._vulnerable = value
 
     @property
-    def declarer(self):
+    def declarer(self) -> str:
         """
         Return the board's declarer as a string.
         Valid values are in the constant SEATS.
@@ -151,7 +147,7 @@ class Board(object):
         return self._declarer
 
     @declarer.setter
-    def declarer(self, value):
+    def declarer(self, value: str):
         """Set the declarer property to an allowed value."""
         if value:
             assert(value in SEATS), 'invalid seat for declarer'
@@ -160,7 +156,7 @@ class Board(object):
         self._declarer = value
 
     @property
-    def declarer_index(self):
+    def declarer_index(self) -> int:  # TODO should be a str
         """
         Return the board's declarer index as an integer.
         Valid values are in the range 0 to 3.
@@ -168,18 +164,18 @@ class Board(object):
         return self._declarer_index
 
     @declarer_index.setter
-    def declarer_index(self, value):
+    def declarer_index(self, value: int):
         """Set the declarer index property to an allowed value."""
         if value in range(4):
             self._declarer_index = value
             self._declarer = SEATS[value]
         else:
             value = -1
-            self._declarer = None
+            self._declarer = ''
         self._declarer_index = value
 
     @property
-    def dealer(self):
+    def dealer(self) -> str:
         """
         Return the board's dealer as a string.
         Valid values are in the constant SEATS.
@@ -187,17 +183,17 @@ class Board(object):
         return self._dealer
 
     @dealer.setter
-    def dealer(self, value):
+    def dealer(self, value: str):
         """Set the dealer property to an allowed value."""
         if value not in SEATS:
             self._dealer_index = -1
-            self._dealer = None
+            self._dealer = ''
             raise ValueError (f'Not a valid seat, {value}')
         self._dealer_index = SEATS.index(value)
         self._dealer = value
 
     @property
-    def dealer_index(self):
+    def dealer_index(self) -> int:
         """
         Return the board's dealer index as an integer.
         Valid values are in the range 0 to 3.
@@ -205,34 +201,34 @@ class Board(object):
         return self._dealer_index
 
     @dealer_index.setter
-    def dealer_index(self, value):
+    def dealer_index(self, value: int):
         """Set the dealer index property to an allowed value."""
         if value in range(4):
             self._dealer = SEATS[value]
         else:
             if value:
-                self._dealer = None
+                self._dealer = ''
                 self._dealer_index = -1
                 raise ValueError('Dealer index not in 0 to 3')
         self._dealer_index = value
 
     @property
-    def auction(self):
+    def auction(self) -> Auction:
         """Return the board's auction as an auction class."""
         return self._auction
 
     @auction.setter
-    def auction(self, value):
+    def auction(self, value: Auction):
         """Set the auction property to an allowed value."""
         self._auction = value
 
     @property
-    def contract(self):
+    def contract(self) -> Contract:
         """Assign contract property."""
         return self._contract
 
     @contract.setter
-    def contract(self, value):
+    def contract(self, value: Contract):
         """Set contract property."""
         self._contract = value
 
@@ -242,22 +238,22 @@ class Board(object):
         return self._hands
 
     @hands.setter
-    def hands(self, value):
+    def hands(self, value: List[Hand]):
         """Set the hand dicts."""
-        self._hands, self._hands_by_index, self._hands_by_seat  = self._set_hands(value)
+        (self._hands, self._hands_by_index, self._hands_by_seat)  = self._set_hands(value)
 
     @property
-    def hands_by_index(self):
+    def hands_by_index(self) -> Dict[int, Hand]:
         """Return a dict of hands keyed by index. 0 is N, etc."""
         return self._hands_by_index
 
     @property
-    def hands_by_seat(self):
+    def hands_by_seat(self) -> Dict[str, Hand]:
         """Return a dict of hands keyed by seat. N, E,  etc."""
         return self._hands_by_seat
 
     @staticmethod
-    def full_pack():
+    def full_pack() -> List[Card]:
         """Return a list containing every card in the pack."""
         pack = []
         for suit in SUITS:
@@ -265,7 +261,7 @@ class Board(object):
         return pack
 
     @staticmethod
-    def build_fourth_hand(existing_hands):
+    def build_fourth_hand(existing_hands: Union[Dict[object, Hand], Dict[int, Hand]]) -> Hand:
         """Build the fourth hand based on the other three hands."""
         used_cards = []
         missing_cards = []
@@ -284,7 +280,7 @@ class Board(object):
         fourth_hand = Hand(missing_cards)
         return fourth_hand
 
-    def board_to_pbn(self):
+    def board_to_pbn(self) -> List[str]:
         """Return a board as a list of strings in pbn format."""
         output = [
             f'[Board "{self._identifier}"]',
@@ -296,7 +292,7 @@ class Board(object):
             output.append(f'[Deal "{self.dealer}:{self.pbn_hands()}"]')
         return output
 
-    def pbn_hands(self, delimiter=' '):
+    def pbn_hands(self, delimiter: str=' ') -> str:
         """Return a board's hands as a string in pbn format."""
         hands_list = []
         dealer_index = SEATS.index(self.dealer)
@@ -315,12 +311,12 @@ class Board(object):
         return delimiter.join(hands_list)
 
     @property
-    def identifier(self):
+    def identifier(self) -> str:
         """Return the board's identifier."""
         return self._identifier
 
     @identifier.setter
-    def identifier(self, value):
+    def identifier(self, value: str):
         """Set the identifier value."""
         if not (isinstance(value, str) or isinstance(value, int)):
             raise TypeError('Identifier must be a string')
