@@ -1,5 +1,7 @@
 """The contract object for the bridgeobjects module."""
 
+from typing import List, Union
+
 from .denomination import Denomination
 from .call import Call
 from .suit import Suit
@@ -27,11 +29,12 @@ class Contract(object):
     It is also identified (if appropriate) as either major, minor or no trumps.
     """
 
-    def __init__(self, name='', declarer='', auction=None):
+    def __init__(self, name: str='', declarer: str='', auction: Union[Auction, None]=None):
         self._name = name
         self._declarer = declarer
         self.auction = self._analyse_auction(auction)
         self._is_nt = False
+        self._denomination = None
         if self._name and self._is_valid(self._name):
             self._call = Call(self._name)
             self._denomination = self._get_denomination(self._name)
@@ -44,46 +47,46 @@ class Contract(object):
         else:
             self._trump_suit = None
             self._call = Call('')
-            self._denomination = None
         if self._name:
             self._level = int(self._name[0])
         else:
             self._level = 0
+        self._modifier  = ''
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """Return the repr string for the object."""
         return f'Contract("{self._call.name}", "{self._declarer}")'
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Return the str string for the object."""
         return f'Contract. {self._call.name} by {self._declarer}'
 
     @property
-    def declarer(self):
+    def declarer(self) -> str:
         """Return the declarer value."""
         return self._declarer
 
     @declarer.setter
-    def declarer(self, value):
+    def declarer(self, value: str):
         """Assign the declarer value."""
         if value and value not in SEATS:
             raise ValueError(f"'{value}' is not a valid seat")
         self._declarer = value
 
     @property
-    def leader(self):
+    def leader(self) -> Union[str, None]:
         """Return the dealer value."""
         if self._declarer:
-           return SEATS[(SEATS.index(self._declarer) + 1) % 4]
+            return SEATS[(SEATS.index(self._declarer) + 1) % 4]
         return None
 
     @property
-    def name(self):
+    def name(self) -> str:
         """Return the name value."""
         return self._name
 
     @name.setter
-    def name(self, value):
+    def name(self, value: str):
         """Assign the name value."""
         if self._is_valid(value):
             self._denomination = self._get_denomination(value)
@@ -94,12 +97,12 @@ class Contract(object):
         self._call = Call(self._name)
 
     @property
-    def call(self):
+    def call(self) -> Call:
         """Return the call value."""
         return self._call
 
     @call.setter
-    def call(self, value):
+    def call(self, value: Union[str, Call]):
         """Assign the denomination value."""
         if isinstance(value, str):
             if value in CONTRACTS:
@@ -113,12 +116,12 @@ class Contract(object):
         self._is_nt = self._denomination.is_nt or self._denomination.is_no_trumps
 
     @property
-    def trump_suit(self):
+    def trump_suit(self) -> Union[Suit, None]:
         """Return a value for the trump suit as a Suit."""
         return self._trump_suit
 
     @trump_suit.setter
-    def trump_suit(self, value):
+    def trump_suit(self, value: Union[str, Suit]):
         """Set the value of the trump suit as a Suit."""
         if isinstance(value, str) and value in SUITS:
             value = SUITS[value]
@@ -128,36 +131,48 @@ class Contract(object):
         self._trump_suit = value
 
     @property
-    def level(self):
+    def level(self) -> int:
         """Return the level value."""
         return self._level
 
     @property
-    def denomination(self):
+    def denomination(self) -> Union[Denomination, None]:
         """Return the denomination value."""
         return self._denomination
 
     @property
-    def is_nt(self):
+    def is_nt(self) -> bool:
         """Return True if the denomination is NT."""
         return self._is_nt
 
+    @property
+    def modifier(self) -> str:
+        """Return contract's modifier (i.e.: '', 'D' or 'R')."""
+        return self._modifier
+
+    @modifier.setter
+    def modifier(self, value: str):  # TODO tests for modifier
+        """Set contract's modifier (i.e.: '', 'D' or 'R')."""
+        if value in ['D', 'R']:
+            self._modifier = value
+        else:
+            self._modifier = ''
+
     @staticmethod
-    def _is_valid(name):
+    def _is_valid(name: str) -> bool:
         """Return True if the contact name is valid."""
         if name not in CONTRACTS:
             raise ValueError(f'{name} is not a valid contract')
         return True
 
     @staticmethod
-    def _get_denomination(name):
+    def _get_denomination(name: str) -> Denomination:
         """Return the denomination of the contract."""
         if name[1:3] == 'NT':
             return Denomination('NT')
-        else:
-            return Denomination(name[1])
+        return Denomination(name[1])
 
-    def _analyse_auction(self, auction):
+    def _analyse_auction(self, auction: Union[Auction, None]) -> Union[Auction, None]:
         """Generate name and declarer from auction and return auction."""
         if auction:
             if auction.calls and auction.first_caller:
@@ -182,7 +197,7 @@ class Contract(object):
         return None
 
     @staticmethod
-    def _three_final_passes(calls):
+    def _three_final_passes(calls: List[Call]) -> bool:
         """Return True if there have been three consecutive passes."""
         three_passes = False
         if len(calls) >= 4:
@@ -191,7 +206,7 @@ class Contract(object):
         return three_passes
 
     @staticmethod
-    def _passed_out(calls):
+    def _passed_out(calls: List[Call]) -> bool:
         """Return True if the board has been passed out."""
         if len(calls) != 4:
             return False
