@@ -1,11 +1,13 @@
 """Provide file load and save functionality for PBN, RBN and lin formats."""
 
-__all__ = ['load_pbn', 'save_pbn', 'load_lin', 'load_rbn', 'parse_pbn', 'parse_lin', 'save_rbn', 'BoardAsString']
+from typing import Dict, List, Union, Tuple
+
+__all__ = ['load_pbn', 'save_pbn', 'pbn_as_text', 'load_lin', 'load_rbn', 'parse_pbn', 'parse_lin', 'save_rbn', 'BoardAsString']
 
 import os
 import datetime
 import re
-import uuid 
+import uuid
 
 from .event import Event
 from .board import Board
@@ -19,14 +21,14 @@ from .auction import Auction
 from .constants import SUIT_NAMES, RANKS, CALLS, SEATS, CONTRACTS, CARD_NAMES
 
 PBN_VULNERABILITY_CONVERSION = {'Z': 'None', 'N': 'NS', 'E': 'EW',
-                                'B': 'Both', '?': None,
+                                'B': 'Both', '?': '',
                                 'None': 'Z', 'NS': 'N', 'EW': 'E',
-                                'Both': 'B', None: '?'}
+                                'Both': 'B', '': '?'}
 
 LIN_VULNERABILITY_CONVERSION = {'0': 'None', 'N': 'NS', 'E': 'EW',
-                                'B': 'Both', '?': None,
+                                'B': 'Both', '?': '',
                                 'None': '0', 'NS': 'N', 'EW': 'E',
-                                'Both': 'B', None: '?'}
+                                'Both': 'B', '': '?'}
 
 PBN_CALL_CONVERSION = {'P': 'Pass', 'D': 'X', 'R': 'XX', 'A': 'AP',
                        'Pass': 'P', 'X': 'D', 'XX': 'R', 'AP': 'A'}
@@ -42,9 +44,9 @@ RBN_CREATOR_TEXT = '% Created by bridgeobjects'
 DEFAULT_EVENT_NAME = 'BfG default event name'
 DEFAULT_EVENT_DATE = datetime.date(1980, 1, 1)
 
-DUMMY_EVENT = uuid.uuid1()
+DUMMY_EVENT = str(uuid.uuid1())
 
-def load_pbn(path):
+def load_pbn(path: str) -> List[Event]:
     """Load a pbn file to a list of events."""
     events = []
     file_text = _get_file_text(path)
@@ -53,7 +55,7 @@ def load_pbn(path):
     return events
 
 
-def save_pbn(events, path, append=False):
+def save_pbn(events: List[Event], path: str, append: bool=False) -> bool:
     """Save a list of events in pbn format."""
     success = False
     output = _create_pbn_list(events, path)
@@ -61,8 +63,13 @@ def save_pbn(events, path, append=False):
         success = bool(_write_file(output, path, append))
     return success
 
+def pbn_as_text(events: List[Event]) -> List[str]:
+    """Return an event in pbn format as a \n  delimited string."""
+    output = _create_pbn_list(events)
+    return output
 
-def load_rbn(path):
+
+def load_rbn(path: str) -> List[Event]:
     """Load a rbn file to a list of events."""
     events = []
     file_text = _get_file_text(path)
@@ -71,7 +78,7 @@ def load_rbn(path):
     return events
 
 
-def save_rbn(events, path, append=False):
+def save_rbn(events: List[Event], path: str, append: bool=False) -> bool:
     """Save a list of events in rbn format."""
     success = False
     output = _create_rbn_list(events)
@@ -80,7 +87,7 @@ def save_rbn(events, path, append=False):
     return success
 
 
-def load_lin(path):
+def load_lin(path: str) -> List[Event]:
     """Load a lin file to a list of events."""
     events = []
     file_text = _get_file_text(path)
@@ -89,38 +96,39 @@ def load_lin(path):
     return events
 
 
-def _get_file_text(path):
+def _get_file_text(path: str) -> List[str]:
     """Return the file text lines as a list."""
-    file_text = ""
+    file_text = []
     try:
         with open(path, "r") as pbn_file:
             raw_text = pbn_file.read()
             raw_text = raw_text.replace(chr(13), '')
             raw_text = raw_text.replace('\r', '')
             file_text = raw_text.split('\n')
-    except FileNotFoundError:
-        raise
+    except FileNotFoundError as error:
+        raise FileNotFoundError(f'{path} is not a file') from error
     return file_text
 
 
-def parse_pbn(file_lines):
+def parse_pbn(file_lines: List[str]) -> List[Event]:
     """Populate bridgeobjects objects from a pbn file."""
     current_name = None
     current_board_identifier = ""
     auction_state = False
     play_state = False
     board_state = False
+    board_index = 0
     events = []
-    event = None
-    board = None
-    value = None
+    event = Event()
+    board = Board()
+    value = ''
     for line in file_lines:
         if line:
             tag = ""
             if line.startswith("%"): # Comments
                 pass
             elif line.startswith("[") or line.startswith("{"):
-                tag, value = _parse_pbn_line(line)
+                (tag, value) = _parse_pbn_line(line)
                 if auction_state and tag != 'Note':
                     auction_state = False
                 elif play_state and tag != 'Note':
@@ -129,7 +137,7 @@ def parse_pbn(file_lines):
                     if board_state and tag == 'Description':
                         board.description = value
                         board_state = False
-                        
+
             # Event event
             if tag == "Event":
                 if not value:
@@ -137,8 +145,9 @@ def parse_pbn(file_lines):
                 if current_name != value:
                     current_name = value
                     current_board_identifier = ""
+                    board_index = 0
                     if value and value != DEFAULT_EVENT_NAME:
-                        event = Event(value)
+                        event = Event(str(value))
                         events.append(event)
 
             # Event location
@@ -156,6 +165,9 @@ def parse_pbn(file_lines):
             # Board identifier
             elif tag == "Board":
                 board_state = True
+                if not value:
+                    board_index += 1
+                    value = str(board_index)
                 board_identifier = value
                 if current_board_identifier != board_identifier:
                     current_board_identifier = board_identifier
@@ -196,7 +208,7 @@ def parse_pbn(file_lines):
 
             # Board hands
             elif tag == "Deal":
-                board.hands = _parse_pbn_deal(board, value)
+                board.hands = _parse_pbn_deal(value)
                 # board.hands_by_seat = {
                 #     'N': board.hands['N'],
                 #     'E': board.hands['E'],
@@ -213,7 +225,7 @@ def parse_pbn(file_lines):
             # Declarer
             elif tag == "Declarer":
                 if value == '?':
-                    value = None
+                    value = ''
                 board.contract.declarer = value
                 board.declarer = value
 
@@ -248,14 +260,14 @@ def parse_pbn(file_lines):
                 play_state = True
             elif play_state:
                 trick = _parse_pbn_play_section(board, tag, value, line, first_player)
-                if trick:
+                if trick.cards:
                     # board.tricks[trick_index] = trick
                     board.tricks.append(trick)
                     trick_index += 1
     return events
 
 
-def _parse_pbn_line(line):
+def _parse_pbn_line(line: str) -> Tuple[str, str]:
     """Return the tag and value from a pbn text line."""
     # example pbn line: [Site "Amsterdam, The Netherlands NLD"]
     space_at = line.find(" ")
@@ -263,26 +275,26 @@ def _parse_pbn_line(line):
     value = line[space_at + 1:]
     value = value[:-1]
     value = value.replace('"', "")
-    return tag, value
+    return (tag, value)
 
 
-def _parse_pbn_date(value):
+def _parse_pbn_date(value: str) -> datetime.date:
     """Return a datetime date object from a pbn date string."""
     # example date value: 2018.02.26
     is_valid = re.search("[0-9]{4}.[\0-9]{2}.[0-9]{2}", value)
     if is_valid:
-        date = value.split('.')
-        date = list(map(int, date))
+        date_str = value.split('.')
+        date = list(map(int, date_str))
         return datetime.date(date[0], date[1], date[2])
     else:
         return DEFAULT_EVENT_DATE
 
 
-def _parse_pbn_deal(board, deal):
+def _parse_pbn_deal(deal: str) -> Dict[object, Hand]:
     """Return a list of hands from a pbn deal string."""
     # example deal value:
     # N:K87.Q642.AJ6.K73 T5.85.Q752.Q6542 AQJ96.73.T843.JT 432.AKJT9.K9.A98
-    hands = {}
+    hands: Dict[object, Hand] = {}
     # Assign hands to board in correct seat; 0 and 'N, 1 and 'E' etc..
     raw_hands = deal[2:].split(" ")
     first_seat = SEATS.index(deal[0])
@@ -295,7 +307,7 @@ def _parse_pbn_deal(board, deal):
     return hands
 
 
-def _parse_pbn_auction(board, tag, value, line):
+def _parse_pbn_auction(board: Board, tag: str, value: str, line: str):
     """Use the tag and value to update the board's auction attributes."""
     if tag == 'Note':
         note_line = value.split(':', 1)
@@ -304,7 +316,7 @@ def _parse_pbn_auction(board, tag, value, line):
         line = ' '.join(line.split())
         auction_line = line.split(' ')
         call_index = len(board.auction.calls) - 1
-        for index, element in enumerate(auction_line):
+        for element in auction_line:
             if element in PBN_CALL_CONVERSION:
                 element = PBN_CALL_CONVERSION[element]
             if element == '-':
@@ -319,29 +331,28 @@ def _parse_pbn_auction(board, tag, value, line):
             elif element[0] == '=':
                 board.auction.note_keys[call_index] = element.replace('=', '')
             else:
-                assert False, f'invalid element in calls: {element}'
+                raise ValueError(f'invalid element in calls: {element}')
 
 
-def _parse_pbn_play_section(board, tag, value, line, first_player):
+def _parse_pbn_play_section(board: Board, tag: str, value: str, line: str, first_player: str) -> Trick:
     """Use the tag and value to update the board's Tricks."""
-    trick = None
+    trick = Trick()
     note_found = False
     note_found = False
-    trick_index = 0
+    card: Union[Card, None] = None
     if tag == 'Note':
         note_line = value.split(':', 1)
         board.play_notes[note_line[0]] = note_line[1]
     else:
         line = ' '.join(line.split())
-        trick = Trick(trick_index)
-        trick_index += 1
+        trick = Trick()
         play_line = line.split(' ')
-        trick_note_keys = []
+        trick_note_keys: List[str] = []
         card_index = 0
         for element in play_line:
             if len(element) == 2:
                 element = ''.join([element[1], element[0]])
-            if element[0] == '=':
+            if element[0] == '=' and card:
                 note_found = True
                 trick_note_keys[card_index-1] = element.replace('=', '')
                 card.play_note_index = element.replace('=', '')
@@ -359,7 +370,7 @@ def _parse_pbn_play_section(board, tag, value, line, first_player):
     return trick
 
 
-def _get_pbn_dealer_index(board, deal):
+def _get_pbn_dealer_index(board: Board, deal: str) -> int:
     """
         Return the first hand index to ensure that the first hand
         assigned to the board's hands list is that of the board dealer.
@@ -376,14 +387,13 @@ def _get_pbn_dealer_index(board, deal):
     return hand_index
 
 
-def parse_rbn(file_text):
+def parse_rbn(file_text: List[str]) -> List[Event]:
     """Populate bridgeobjects objects from a rbn file."""
     events = []
-    current_name = ""
     current_board_identifier = ""
     create_event = True
-    board = None
-    north, south, west, east = "", "", "", ""
+    board = Board()
+    (north, south, west, east) = ("", "", "", "")
     for line in file_text:
         if line:
             tag = ""
@@ -433,7 +443,7 @@ def parse_rbn(file_text):
                 # Board dealer and hands
                 elif tag == "H":
                     board.dealer = value[0]
-                    board.hands = _parse_rbn_deal(board, value)
+                    board.hands = _parse_rbn_deal(value)
 
                 # Board auction and vulnerable
                 elif tag == "A":
@@ -445,32 +455,33 @@ def parse_rbn(file_text):
                     raw_data = value.split(':')
                     board.contract = _get_rbn_contract(raw_data[0])
                     if raw_data[1] == '?':
-                        declarer = None
+                        declarer = ''
                     else:
                         declarer = raw_data[1]
                     board.contract.declarer = declarer
 
                 # Board players
                 elif tag == "N":
-                    north, south, west, east = _parse_rbn_seats(value)
-                    if board:
-                        board.north, board.south, board.west, board.east = \
-                            north, south, west, east
+                    (north, south, west, east) = _parse_rbn_seats(value)
+                    (board.north, board.south, board.west, board.east) = \
+                        (north, south, west, east)
         else:
             create_event = True
     return events
 
 
-def _parse_rbn_deal(board, deal):
+def _parse_rbn_deal(deal: str) -> Dict[object, Hand]:
     """Return a list of hands from a rbn deal string."""
     # example deal value:
     # W:A8765.QT.K9.AT87:J42.AJ7632.J.632:QT3.85.Q86.KQJ54:
-    hands = {}
+    hands: Dict[object, Hand] = {}
     raw_hands = deal[2:].split(":")
     if len(raw_hands[3]) == 0:
-        raw_hands = _build_fourth_rpn_hand(raw_hands)
+        hand_list = _build_fourth_rpn_hand(raw_hands)
+    else:
+        hand_list = raw_hands
     first_seat = SEATS.index(deal[0])
-    for index, card_list in enumerate(raw_hands):
+    for index, card_list in enumerate(hand_list):
         seat_index = (first_seat + index) % 4
         seat_name = SEATS[seat_index]
         hand = Hand(card_list)
@@ -479,7 +490,7 @@ def _parse_rbn_deal(board, deal):
     return hands
 
 
-def _parse_rbn_date(value):
+def _parse_rbn_date(value: str) -> datetime.date:
     """Return a datetime date object from a pbn date string."""
     # example date value: 19930512
     year = int(value[:4])
@@ -488,8 +499,8 @@ def _parse_rbn_date(value):
     return datetime.date(year, month, day)
 
 
-def _parse_rbn_seats(value):
-    """Return, north, south, west, east for a rbn file."""
+def _parse_rbn_seats(value: str) -> Tuple[str, str, str, str]:
+    """Return, north, south, west, east for an rbn file."""
     north, south, west, east = "", "", "", ""
     if value:
         pairs = value.split(":")
@@ -505,13 +516,23 @@ def _parse_rbn_seats(value):
                 west = west_east[0]
             if len(west_east) >= 2:
                 east = west_east[1]
-    return north, south, west, east
+    return (north, south, west, east)
 
 
-def _build_fourth_rpn_hand(raw_hands):
+def _build_fourth_rpn_hand(raw_hands: List[str]) -> List[str]:
     """Build the fourth hand based on the other three hands."""
-    used_cards = [[], [], [], []]
-    last_hand = [[], [], [], []]
+    used_cards: Dict[int, List[str]] = {
+        0: [],
+        1: [],
+        2: [],
+        3: [],
+    }
+    last_hand: Dict[int, List[str]] = {
+        0: [],
+        1: [],
+        2: [],
+        3: [],
+    }
     # Create a list of cards used by other three hands.
     for hand in raw_hands[:-1]:
         suit_cards = hand.split('.')
@@ -525,19 +546,20 @@ def _build_fourth_rpn_hand(raw_hands):
                 last_hand[suit].append(rank)
     # Convert list to string form.
     suit_names = []
-    for suit in last_hand:
-        suit_names.append(('').join(suit))
+    for index in range(4):
+        suit_cards = last_hand[index]
+        suit_names.append(('').join(suit_cards))
     fourth_hand = ('.').join(suit_names)
     raw_hands[3] = fourth_hand
     return raw_hands
 
 
-def _get_rbn_vulnerable(value):
+def _get_rbn_vulnerable(value: str) -> str:
     """Return the auction vulnerability as a string."""
     return PBN_VULNERABILITY_CONVERSION[value[1]]
 
 
-def _parse_rbn_auction(value):
+def _parse_rbn_auction(value: str) -> Auction:
     """Return the auction as a list of board.CALLS."""
     auction = Auction()
     pointer = 0
@@ -564,19 +586,16 @@ def _parse_rbn_auction(value):
     return auction
 
 
-def _get_rbn_contract(value):
+def _get_rbn_contract(value: str) -> Contract:
     """Return the contract."""
     contract = Contract(name=value)
-    # if len(value) > 3:
-    #     value = value[:3]
-    # if value in CONTRACTS:
-    #     contract.name = value
     return contract
 
 
-def _create_pbn_list(events, path=''):
+def _create_pbn_list(events: List[Event], path: str='') -> List[str]:
     """Create a list of strings for events in pbn format."""
     output = []
+    output = _append_pbn_header()
     if path:
         if not os.path.isfile(path):
             output = _append_pbn_header()
@@ -596,7 +615,7 @@ def _create_pbn_list(events, path=''):
     return output
 
 
-def _append_pbn_header():
+def _append_pbn_header() -> List[str]:
     """Create a list containing the pbn file header."""
     output = []
     output.append(PBN_VERSION_TEXT)
@@ -607,7 +626,7 @@ def _append_pbn_header():
     return output
 
 
-def _create_pbn_event(event):
+def _create_pbn_event(event: Event) -> List[str]:
     """Return an event as string in pbn format as a list."""
     output = []
     output.append(f'[Event "{event.name}"]')
@@ -654,7 +673,7 @@ def _create_pbn_event(event):
 #     return output
 
 
-def _create_pbn_auction(board):
+def _create_pbn_auction(board: Board) -> List[str]:
     """Return an auction as a pbn list."""
     auction_list = ['[Auction "{}"]'.format(board.contract.declarer)]
     dealer_index = SEATS.index(board.dealer)
@@ -674,12 +693,12 @@ def _create_pbn_auction(board):
     return auction_list
 
 
-def _create_pbn_play_section(board):
+def _create_pbn_play_section(board: Board) -> List[str]:
     """Return board's tricks as a pbn list."""
     trick_list = []
     player = board.tricks[0].leader
     trick_list.append('[Play "{}"]'.format(player))
-    for index, trick in sorted(board.tricks.items()):
+    for trick in board.tricks:
         if trick.leader:
             line = []
             for index, card in enumerate(trick.cards):
@@ -691,7 +710,7 @@ def _create_pbn_play_section(board):
     return trick_list
 
 
-def _question_from_null(value):
+def _question_from_null(value: str) -> str:
     """Return a question mark if value is None or ''."""
     if not value:
         return "?"
@@ -699,7 +718,7 @@ def _question_from_null(value):
         return value
 
 
-def _get_pbn_deal(board, delimiter=' '):
+def _get_pbn_deal(board: Board, delimiter: str=' ') -> str:
     """Return a board's hands as a string in pbn format."""
     hands_list = []
     dealer_index = SEATS.index(board.dealer)
@@ -718,7 +737,7 @@ def _get_pbn_deal(board, delimiter=' '):
     return delimiter.join(hands_list)
 
 
-def _create_rbn_list(events):
+def _create_rbn_list(events: List[Event]) -> List[str]:
     """Create a list of strings for events in rbn format."""
     output = []
     output.append(RBN_VERSION_TEXT)
@@ -728,7 +747,7 @@ def _create_rbn_list(events):
     return output
 
 
-def _create_rbn_event(event):
+def _create_rbn_event(event: Event) -> List[str]:
     """Return an event as string in rbn format as a list."""
     output = []
     output.append(f'E {event.name}')
@@ -738,11 +757,11 @@ def _create_rbn_event(event):
         output.append(f'D {event_date}')
     output.append(f'M {_question_from_null(event.scoring_method)}')
     for board in event.boards:
-        output.extend(_create_rbn_board(event, board))
+        output.extend(_create_rbn_board(board))
     return output
 
 
-def _create_rbn_board(event, board):
+def _create_rbn_board(board: Board) -> List[str]:
     """Return a board as a list of strings in pbn format."""
     output = [f'B {board.identifier}']
 
@@ -760,9 +779,9 @@ def _create_rbn_board(event, board):
     output.append('A {}{}:'.format(board.dealer, vulnerable))
 
     # contract
-    board_contract = _question_from_null(board.contract)
+    board_contract = _question_from_null(board.contract.name)
     board_declarer = _question_from_null(board.declarer)
-    contract = ':'.join([board_contract.name, board_declarer])
+    contract = ':'.join([board_contract, board_declarer])
     if contract != "?:?":
         output.append(f'C {contract}')
 
@@ -774,7 +793,7 @@ def _create_rbn_board(event, board):
     return output
 
 
-def _create_rbn_names(board):
+def _create_rbn_names(board: Board) -> str:
     """Return the boards players in rbn format."""
     names = '{}+{}:{}+{}'.format(board.north, board.south, board.west, board.east)
     if names == "+:+":
@@ -786,26 +805,25 @@ def _create_rbn_names(board):
     return names
 
 
-def _write_file(output, path, append):
+def _write_file(output: List[str], path: str, append: bool) -> bool:
     """Write the list 'output' to a text file defined by path."""
     if not os.path.isfile(path):
-        with open(path, "w") as pbn_file:
-            pbn_file.write('')
+        with open(path, 'w') as f_clear_pbn_file:
+            f_clear_pbn_file.write('')
     try:
         if append:
             mode = 'a'
         else:
             mode = 'w'
-        with open(path, mode) as pbn_file:
-            pbn_file.write('\n'.join(output))
-        success = True
-    except FileNotFoundError:
-        logger.warning(f'invalid file path: {path}. File not written')
-        success = False
-    return success
+        with open(path, mode) as f_pbn_file:
+            f_pbn_file.write('\n'.join(output))
+        return True
+    except FileNotFoundError as error:
+        raise FileNotFoundError(f'invalid file path: {path}. File not written') from error
+    return False
 
-
-def parse_lin(file_text):
+# TODO test for lin event
+def parse_lin(file_text: List[str]) -> List[Event]:
     """Populate bridgeobjects objects from a lin file."""
     events = []
     event = Event()
@@ -827,7 +845,7 @@ def parse_lin(file_text):
     return events
 
 
-def _parse_lin_board(line):
+def _parse_lin_board(line: str) -> Board:
     """Return a board object from a lin line."""
     board = Board()
     line_list = line.split('|')
@@ -839,13 +857,13 @@ def _parse_lin_board(line):
     return board
 
 
-def _parse_lin_hands(line_list, dealer):
+def _parse_lin_hands(cards: str, dealer: str) -> Dict[object, Hand]:
     """Return a list of hands from lin text."""
-    hands = {}
+    hands: Dict[object, Hand] = {}
     dealer_index = SEATS.index(dealer)
     hand_index = (2 - dealer_index) % 4
     hand = Hand()
-    for character in line_list:
+    for character in cards:
         if character in Suit.SHORT_NAMES:
             suit_name = character
         elif character == ',':
@@ -873,7 +891,7 @@ class BoardAsString(object):
         pass
 
     @staticmethod
-    def create_pbn_board(event, board):
+    def create_pbn_board(event: Event, board: Board) -> List[str]:
         """Return a board as a list of strings in pbn format."""
         output = [f'[Board "{board.identifier}"]',
                   f'[West "{board.west}"]',
@@ -905,4 +923,3 @@ class BoardAsString(object):
                 output.append(note)
             output.append('')
         return output
-
