@@ -1,9 +1,11 @@
 """The Hand class represents a dealt hand of thirteen cards."""
 
+from typing import Dict, List, Union
+
 
 from .card import Card
 from .denomination import Denomination
-from .suit import Suit, Spades, Hearts, Diamonds, Clubs
+from .suit import Suit
 from .constants import (SUIT_NAMES, RANKS, SUITS, HONOUR_POINTS,
                         BALANCED_SHAPES, SEMI_BALANCED_SHAPES,
                         CARD_NAMES, DISPLAY_SUIT_ORDER)
@@ -44,7 +46,7 @@ class Hand(object):
     CANDIDATES_4441 = {'S': SUITS['H'], 'H': SUITS['S'], 'D': SUITS['H'], 'C': SUITS['S']}
 
 
-    def __init__(self, cards=None):
+    def __init__(self, cards: Union[List[Card], str, None]=None):
         """
         Create a list of cards from the input parameter and
         check that the hand is valid and assign properties.
@@ -55,7 +57,7 @@ class Hand(object):
 
         # attributes set once the cards are known for the hand
         self._high_card_points = 0
-        self._shape = []
+        self._shape: List[int] = []
         self._equal_long_suits = False
         self._suits_by_length = None
         self._longest_suit = None
@@ -79,16 +81,19 @@ class Hand(object):
 
         # set the hand's minor properties
         # These are initialised to None, and evaluated the first time they are required.
-        self._cards_by_suit = None
-        self._distribution_points = None
+        self._cards_by_suit: Dict[str, List[Card]] = {}
+        self._distribution_points = 0
         self._is_balanced = None
         self._is_semi_balanced = None
         self._rule_of_nineteen = None
         self._rule_of_twenty = None
 
-        self._cards = self._set_cards(cards)
+        if cards:
+            self._cards = self._set_cards(cards)
+        else:
+            self._cards = []
 
-    def _set_cards(self, cards):
+    def _set_cards(self, cards: Union[List[Card], str]) -> List[Card]:
         """Return the hand's cards as a list of Card objects."""
         card_list = []
         if cards:
@@ -128,15 +133,15 @@ class Hand(object):
         self._third_suit = self._get_third_suit()
         self._is_valid = self._check_is_valid()
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return f'Hand("{self._get_pbn_string()}")'
 
-    def __str__(self):
+    def __str__(self) -> str:
         sorted_cards = sorted(self._cards, key=lambda x: x.order, reverse=True)
         cards = [card.name for card in sorted_cards]
         return f'Hand({", ".join(cards)})'
 
-    def __eq__(self, other):
+    def __eq__(self, other) -> bool:
         """Provide eq functionality for hands"""
         if self._get_pbn_string() == other:
             return True
@@ -144,7 +149,7 @@ class Hand(object):
             return False
 
     @property
-    def cards(self):
+    def cards(self) -> List[Card]:
         """
         The hand's cards.
 
@@ -153,13 +158,13 @@ class Hand(object):
         return self._cards
 
     @cards.setter
-    def cards(self, value):
+    def cards(self, value: Union[List[Card], str]):
         """Sets the hand's cards."""
         # value can be a list of card names, a list of Card objects or a PBN string.
         self._cards = self._set_cards(value)
 
     @property
-    def is_valid(self):
+    def is_valid(self) -> bool:
         """
         Indicates whether or not the hand is valid.
 
@@ -169,127 +174,127 @@ class Hand(object):
         return self._is_valid
 
     @property
-    def spades(self):
+    def spades(self) -> int:
         """Return the number of spade cards in the hand as an integer."""
         return self._spades
 
     @property
-    def hearts(self):
+    def hearts(self) -> int:
         """Return the number of heart cards in the hand as an integer."""
         return self._hearts
 
     @property
-    def diamonds(self):
+    def diamonds(self) -> int:
         """Return the number of diamond cards in the hand as an integer."""
         return self._diamonds
 
     @property
-    def clubs(self):
+    def clubs(self) -> int:
         """Return the number of club cards in the hand as an integer."""
         return self._clubs
 
     @property
-    def aces(self):
+    def aces(self) -> int:
         """Return the number of aces in the hand as an integer."""
         return self._aces
 
     @property
-    def kings(self):
+    def kings(self) -> int:
         """Return the number of kings in the hand as an integer."""
         return self._kings
 
     @property
-    def queens(self):
+    def queens(self) -> int:
         """Return the number of queens in the hand as an integer."""
         return self._queens
 
     @property
-    def jacks(self):
+    def jacks(self) -> int:
         """Return the number of jacks in the hand as an integer."""
         return self._jacks
 
     @property
-    def tens(self):
+    def tens(self) -> int:
         """Return the number of tens in the hand as an integer."""
         return self._tens
 
     @property
-    def nines(self):
+    def nines(self) -> int:
         """Return the number of nines in the hand as an integer."""
         return self._nines
 
     @property
-    def tens_and_nines(self):
+    def tens_and_nines(self) -> int:
         """Returns a count of the tens + nines in the hand."""
         return self._tens_and_nines
 
     @property
-    def seven_card_suit_or_better(self):
+    def seven_card_suit_or_better(self) -> bool:
         """Return True if the longest suit has seven or more cards."""
         return self._shape[0] >= 7
 
     @property
-    def seven_card_suit(self):
+    def seven_card_suit(self) -> bool:
         """Return True if the longest suit has precisely seven cards."""
         return 7 in self._shape
 
     @property
-    def six_card_suit_or_better(self):
+    def six_card_suit_or_better(self) -> bool:
         """Return True if the longest suit has six or more cards."""
         return self._shape[0] >= 6
 
     @property
-    def six_card_suit(self):
+    def six_card_suit(self) -> bool:
         """Return True if the longest suit has precisely six cards."""
         return 6 in self._shape
 
     @property
-    def five_card_suit_or_better(self):
+    def five_card_suit_or_better(self) -> bool:
         """Return True if the longest suit has five or more cards."""
         return self._shape[0] >= 5
 
     @property
-    def five_card_suit(self):
+    def five_card_suit(self) -> bool:
         """Return True if the longest suit has precisely five cards."""
         return 5 in self._shape
 
     @property
-    def five_card_major_or_better(self):
+    def five_card_major_or_better(self) -> bool:
         """Return True if the hand has a major suit with five or more cards."""
         return self._spades >= 5 or self._hearts >= 5
 
     @property
-    def five_card_major(self):
+    def five_card_major(self) -> bool:
         """Return True if the hand has a major suit with precisely 5 cards."""
         return self._spades == 5 or self._hearts == 5
 
     @property
-    def five_card_major_suit(self):
+    def five_card_major_suit(self) -> Suit:
         """Return the suit if the hand has a major suit with 5 or more cards."""
         return self._get_major_suit(5)
 
     @property
-    def four_card_major_or_better(self):
+    def four_card_major_or_better(self) -> bool:
         """Return True if the hand has a major suit with four or more cards."""
         return self._spades >= 4 or self._hearts >= 4
 
     @property
-    def four_card_major(self):
+    def four_card_major(self) -> bool:
         """Return True if the hand has a major suit with 4 or more cards."""
         return self._spades >= 4 or self._hearts >= 4
 
     @property
-    def four_card_major_suit(self):
+    def four_card_major_suit(self) -> Suit:
         """Return the suit if the hand has a major suit with 4 or more cards."""
         return self._get_major_suit(4)
 
     @property
-    def equal_long_suits(self):
+    def equal_long_suits(self) -> bool:
         """Return True if the two longest suits are the same length."""
         return self._equal_long_suits
 
     @property
-    def shape(self):
+    def shape(self) -> List[int]:
         """Return the shape of the hand as a list in descending
            order by holding e.g. [4, 3, 3, 3]."""
         if len(self._shape) != 4:
@@ -297,7 +302,7 @@ class Hand(object):
             self._shape = self._get_shape()
         return self._shape
 
-    def suit_length(self, suit):
+    def suit_length(self, suit: Union[Suit, str]) -> int:
         """Return the length of a suit."""
         if isinstance(suit, str):
             suit = Suit(suit)
@@ -307,14 +312,14 @@ class Hand(object):
             raise TypeError("suit must be a suit instance")
 
     @property
-    def suits_by_length(self):
+    def suits_by_length(self) -> Union[List[Suit], None]:
         """Return the list of hand's suits, in descending order by holding."""
         if self._suits_by_length is None:
             self._suits_by_length = self._get_suits_by_length()
         return self._suits_by_length
 
     @property
-    def longest_suit(self):
+    def longest_suit(self) -> Union[Suit, None]:
         """
         Return the longest suit in the hand as a Suit object.
 
@@ -332,7 +337,7 @@ class Hand(object):
         return self._longest_suit
 
     @property
-    def second_suit(self):
+    def second_suit(self) -> Union[Suit, None]:
         """
         Return the second longest suit in the hand as a Suit object.
 
@@ -348,23 +353,23 @@ class Hand(object):
         return self._second_suit
 
     @property
-    def third_suit(self):
+    def third_suit(self) -> Union[Suit, None]:
         """Return the third longest suit in the hand as a Suit object."""
         return self._third_suit
 
     @property
-    def distribution_points(self):
+    def distribution_points(self) -> int:
         """
         Return the distribution points count for the hand.
 
         Add 3 for a void, 2 for a singleton and 1 for a doubleton.
         """
-        if self._distribution_points is None:
+        if not self._distribution_points:
             self._distribution_points = self._get_distribution_points()
         return self._distribution_points
 
     @property
-    def high_card_points(self):
+    def high_card_points(self) -> int:
         """
         Return the (Milton Work) high card points for the hand.
 
@@ -373,86 +378,78 @@ class Hand(object):
         return self._high_card_points
 
     @property
-    def hcp(self):
+    def hcp(self) -> int:
         """An alias for high_card_points."""
         return self._high_card_points
 
     @property
-    def four_four_four_one(self):
+    def four_four_four_one(self) -> bool:
         """Return True if the hand shape is [4, 4, 4, 1]."""
         return self._shape == [4, 4, 4, 1]
 
     @property
-    def five_four_or_better(self):
+    def five_four_or_better(self) -> bool:
         """Return True if hand contains 5 card suit and a 4 card suit or better."""
         return self.shape[0] >= 5 and self.shape[1] >= 4
 
     @property
-    def five_five_or_better(self):
+    def five_five_or_better(self) -> bool:
         """Return True if hand contains two five card suits or better."""
         return self.shape[0] >= 5 and self.shape[1] >= 5
 
     @property
-    def five_four(self):
+    def five_four(self) -> bool:
         """Return True if hand contains a 5 card suit and a 4 card suit."""
         return self.shape[0] >= 5 and self.shape[1] == 4
 
     @property
-    def five_five(self):
+    def five_five(self) -> bool:
         """Return True if hand contains two five card suits."""
         return self.shape[0] >= 5 and self.shape[1] >= 5
 
     @property
-    def six_four(self):
+    def six_four(self) -> bool:
         """Return True if hand contains a 6 and a 4 card suit."""
         return self.shape[0] >= 6 and self.shape[1] >= 4
 
     @property
-    def six_six(self):
+    def six_six(self) -> bool:
         """Return True if hand contains two 6 card suits."""
         return self.shape[0] >= 6 and self.shape[1] >= 6
 
     @property
-    def is_balanced(self):
+    def is_balanced(self) -> bool:
         """Return True if 'shape' matches one of the balanced shapes."""
-        if self._is_balanced is None:
-            self._is_balanced = self._get_is_balanced()
-        return self._is_balanced
+        return self._get_is_balanced()
 
     @property
-    def is_semi_balanced(self):
+    def is_semi_balanced(self) -> bool:
         """
             Return True if 'shape' matches one of the balanced or
             semi-balanced shapes.
         """
-        if self._is_semi_balanced is None:
-            self._is_semi_balanced = self._get_is_semi_balanced()
-        return self._is_semi_balanced
+        return self._get_is_semi_balanced()
 
     @property
-    def rule_of_nineteen(self):
+    def rule_of_nineteen(self) -> bool:
         """Return True if the hand fits the rule on nineteen.
 
            I.e. if the sum of the cards in the two longest suits, plus
            the high card points is greater than or equal to nineteen.
         """
-        if self._rule_of_nineteen is None:
-            self._rule_of_nineteen = self._get_rule_of_n(19)
-        return self._rule_of_nineteen
+        return self._get_rule_of_n(19)
 
     @property
-    def rule_of_twenty(self):
+    def rule_of_twenty(self) -> bool:
         """Return True if the hand fits the rule of twenty.
 
            I.e. if the sum of the cards in the two longest suits, plus
            the high card points is greater than or equal to twenty.
         """
-        if self._rule_of_twenty is None:
-            self._rule_of_twenty = self._get_rule_of_n(20)
-        return self._rule_of_twenty
+        return self._get_rule_of_n(20)
 
     @property
-    def suit_holding(self):
+    def suit_holding(self) -> Dict[object, int]:
         """Return a dict holding the number of spades etc.."""
         return self._suit_holding()
 
@@ -468,14 +465,14 @@ class Hand(object):
                 'C': self._clubs}
 
     @property
-    def cards_by_suit(self):
+    def cards_by_suit(self) -> Dict[str, List[Card]]:
         """Return a dict holding the cards by suit."""
         if not self._cards_by_suit:
             self._cards_by_suit = self._get_cards_by_suit()
         return self._cards_by_suit
 
     @property
-    def honours(self):
+    def honours(self) -> Dict[str, int]:
         """Return a dict of the count of honours  by suit name."""
         honours = {}
         honour_names = 'AKQJT'
@@ -488,12 +485,12 @@ class Hand(object):
         return honours
 
     @property
-    def suit_order(self):
+    def suit_order(self) -> str:
         """Return the value of suit order as a string."""
         return self._suit_order
 
     @suit_order.setter
-    def suit_order(self, value):
+    def suit_order(self, value: str):
         """Set the value of suit order as a string."""
         if not isinstance(value, str):
             raise TypeError('Suit order must be a string')
@@ -507,18 +504,18 @@ class Hand(object):
         self._suit_order = value
 
     @property
-    def high_card_left(self):
+    def high_card_left(self) -> bool:
         """Return the value of high card left as boolean."""
         return self._high_card_left
 
     @high_card_left.setter
-    def high_card_left(self, value):
+    def high_card_left(self, value: bool):
         """Set the boolean value of high card left."""
         if not isinstance(value, bool):
             raise TypeError('Value must be a boolean')
         self._high_card_left = value
 
-    def sorted_cards(self, suit_order=None, high_card_left=None):
+    def sorted_cards(self, suit_order='', high_card_left=None):  # TODO sort out high_card_left here
         """Return the card list sorted according to the suit order and high card left."""
         if not suit_order:
             suit_order = self._suit_order
@@ -631,6 +628,7 @@ class Hand(object):
         suits_by_length = []
         key = lambda item: (item[1], item[0])
         for suit, holding in sorted(holdings.items(), key=key, reverse=True):
+            del holding
             suits_by_length.append(suit)
         return suits_by_length
 
@@ -804,10 +802,10 @@ class Hand(object):
             return True
         return False
 
-    def _get_suit_points(self, suit):
+    def _get_suit_points(self, suit: Union[Suit, Denomination, str]) -> int:
         """Return the high card points for the suit."""
         suit_points = 0
-        if isinstance(suit, Suit) or isinstance(suit, Denomination):
+        if isinstance(suit, (Suit, Denomination)):
             suit_name = suit.name
         else:
             suit_name = suit
@@ -817,7 +815,7 @@ class Hand(object):
                     suit_points += card.value-9
         return suit_points
 
-    def _get_rule_of_fourteen(self, suit):
+    def _get_rule_of_fourteen(self, suit: Suit) -> bool:
         """Return True if hand number of cards plus the suit points equals or exceeds 14 for the given suit'."""
         suit_cards = 0
         for card in self._cards:
@@ -825,7 +823,7 @@ class Hand(object):
                 suit_cards += 1
         return self.suit_points(suit) + suit_cards >= 14
 
-    def _get_solid_suit_honours(self, suit):
+    def _get_solid_suit_honours(self, suit: Suit) -> bool:
         """Return True if the the hand has A, K, Q, J. in 'suit'"""
         honours = 0
         for card in self._cards:
@@ -834,7 +832,7 @@ class Hand(object):
                     honours += 1
         return honours == 4
 
-    def _get_pbn_string(self):
+    def _get_pbn_string(self) -> str:
         """Return hand as a string in pbn format."""
         hand_list = []
         for suit in SUIT_NAMES[::-1]:
@@ -847,23 +845,34 @@ class Hand(object):
         return '.'.join(hand_list)
 
     @staticmethod
-    def sort_card_list(cards, suit_order=None, high_card_left=True):
+    def sort_card_list(cards: List[Card], suit_order: str='', high_card_left: bool=True) -> list[Card]:
         """Return a list of cards sorted by suit and rank."""
         if not suit_order:
             suit_order = DISPLAY_SUIT_ORDER
-        raw_cards = [None]*52
-        for card in cards:
-            suit = suit_order.index(card.suit.name)
-            if high_card_left:
-                index = 13 * suit + 13 - card.value
-            else:
-                index = 13 * suit + card.value - 1
-            raw_cards[index] = card
-        return [card for card in raw_cards if card]
+        card_names = Hand._order_card_names(suit_order, high_card_left)
+        sorted_cards = []
+        for card_name in card_names:
+            card = Card(card_name)
+            if card in cards:
+                sorted_cards.append(card)
+        return sorted_cards
 
-    def _get_cards_by_suit(self):
+    @staticmethod
+    def _order_card_names(suit_order: str, high_card_left: bool) -> List[str]:
+        """Return a list of cards."""
+        card_names = []
+        if high_card_left:
+            ranks = RANKS[:0:-1]
+        else:
+            ranks = RANKS[1:]
+        for suit in suit_order:
+            for rank in ranks:
+                card_names.append(''.join([rank, suit]))
+        return card_names
+
+    def _get_cards_by_suit(self) -> Dict[str, List[Card]]:
         """Return a dict by suit with cards for suit in descending order."""
-        card_suits = {}
+        card_suits: Dict[str, List[Card]] = {}
         for suit_name in SUIT_NAMES:
             card_suits[suit_name] = []
             for card in self._cards:
@@ -871,19 +880,19 @@ class Hand(object):
                     card_suits[suit_name].append(card)
         return card_suits
 
-    def honour_sequences(self):
+    def honour_sequences(self) -> Dict[str, List[Card]]:
         """Return a dict of honour sequences by suit (if any)."""
         honour_sequences = self._touching_honours_and_sequences(min_length=3)
         return honour_sequences
 
-    def touching_honours(self):
+    def touching_honours(self) -> Dict[str, List[Card]]:
         """Return a dict of touching honours by suit (if any)."""
         touching_honours = self._touching_honours_and_sequences(max_length=2)
         return touching_honours
 
-    def _touching_honours_and_sequences(self, min_length=2, max_length=13):
+    def _touching_honours_and_sequences(self, min_length: int=2, max_length: int=13) -> Dict[str, List[Card]]:
         """Return a dict of honour sequences by suit (if any)."""
-        sequences = {}
+        sequences: Dict[str, List[Card]] = {}
         for suit in Suit.SHORT_NAMES:
             sequences[suit] = []
             sequence_length = 1
@@ -905,7 +914,7 @@ class Hand(object):
                 sequences[key] = []
         return sequences
 
-    def internal_sequences(self):
+    def internal_sequences(self) -> List[Card]:
         """Return a list of internal sequences (if any)."""
         internal_sequences = []
         cards = self.cards
